@@ -85,7 +85,7 @@ class KarakeepAPI:
     """
 
     # Version reflects the client library version, updated by bumpver
-    VERSION: str = "0.1.6"
+    VERSION: str = "0.2.2"
 
     def __init__(
         self,
@@ -95,6 +95,7 @@ class KarakeepAPI:
         verify_ssl: bool = True,
         verbose: bool = False,
         disable_response_validation: Optional[bool] = None,
+        rate_limit: float = 0.0,  # Minimum interval between API calls in seconds
     ):
         """
         Initialize the Karakeep API client.
@@ -114,6 +115,9 @@ class KarakeepAPI:
             disable_response_validation: If True, skip Pydantic validation of API responses and return raw data.
                                          Defaults to False. Can be overridden by setting the
                                          KARAKEEP_PYTHON_API_DISABLE_RESPONSE_VALIDATION environment variable to "true".
+            rate_limit: The minimum time interval in seconds between consecutive API calls.
+                        Defaults to 0.0 (no explicit rate limiting).
+                        Can be overridden with KARAKEEP_PYTHON_API_RATE_LIMIT environment variable.
         """
         # --- API Key Validation ---
         resolved_api_key = api_key or os.environ.get("KARAKEEP_PYTHON_API_KEY")
@@ -204,7 +208,31 @@ class KarakeepAPI:
 
         self.verify_ssl = verify_ssl
         self.verbose = verbose
-        self.last_request_time: float = time.monotonic()  # Initialize timestamp for rate limiting
+
+        # --- Rate Limit Setting ---
+        # Argument takes precedence over environment variable
+        env_rate_limit_str = os.environ.get("KARAKEEP_PYTHON_API_RATE_LIMIT")
+        if rate_limit != 0.0:  # Argument provided and not default
+            self.rate_limit = rate_limit
+            logger.debug(f"Rate limit set to {self.rate_limit}s via argument.")
+        elif env_rate_limit_str is not None:
+            try:
+                self.rate_limit = float(env_rate_limit_str)
+                logger.debug(
+                    f"Rate limit set to {self.rate_limit}s via KARAKEEP_PYTHON_API_RATE_LIMIT environment variable."
+                )
+            except ValueError:
+                self.rate_limit = 0.0  # Default if env var is invalid
+                logger.warning(
+                    f"Invalid value for KARAKEEP_PYTHON_API_RATE_LIMIT ('{env_rate_limit_str}'). Rate limiting disabled (0.0s)."
+                )
+        else:
+            self.rate_limit = rate_limit  # Use default from signature (0.0)
+            logger.debug(f"Rate limit set to default: {self.rate_limit}s.")
+
+        self.last_request_time: float = (
+            time.monotonic()
+        )  # Initialize timestamp for rate limiting
 
         # --- Response Validation Setting ---
         # Argument takes precedence over environment variable
@@ -233,6 +261,7 @@ class KarakeepAPI:
         logger.debug(
             f"  Disable Response Validation: {self.disable_response_validation}"
         )
+        logger.debug(f"  Rate Limit: {self.rate_limit}s")
         # API Key is intentionally not logged for security
 
         # --- Initial Connection Check ---
@@ -264,7 +293,7 @@ class KarakeepAPI:
             Union[BaseModel, dict, list, str, bytes]
         ] = None,  # More specific type hint
         extra_headers: Optional[Dict[str, str]] = None,
-    ) -> Any:
+    ) -> Union[Dict[str, Any], List[Any], None]:
         """
         Internal method to make an HTTP call to the Karakeep API. Handles authentication,
         request formatting, response parsing, and error handling.
@@ -578,25 +607,28 @@ class KarakeepAPI:
             ) from e
 
     @optional_typecheck
-    def _enforce_rate_limit(self, min_interval_sec: float = 2.0) -> None:
+    def _enforce_rate_limit(self) -> None:
         """
-        Ensures a minimum time interval between consecutive API calls.
-
-        If the time since the last call is less than `min_interval_sec`, this method
-        will sleep for the remaining duration. It then updates the timestamp of the
-        last request.
-
-        Args:
-            min_interval_sec: The minimum desired interval between requests in seconds.
+        Ensures a minimum time interval between consecutive API calls, based on `self.rate_limit`.
+        If `self.rate_limit` is 0 or less, this method does nothing.
+        Otherwise, if the time since the last call is less than `self.rate_limit`,
+        this method will sleep for the remaining duration. It then updates the
+        timestamp of the last request.
         """
+        if self.rate_limit <= 0:
+            # Update last request time even if rate limiting is disabled or not triggered,
+            # so the next call has an accurate timestamp if rate_limit is changed.
+            self.last_request_time = time.monotonic()
+            return
+
         current_time = time.monotonic()
         time_since_last = current_time - self.last_request_time
 
-        if time_since_last < min_interval_sec:
-            sleep_duration = min_interval_sec - time_since_last
+        if time_since_last < self.rate_limit:
+            sleep_duration = self.rate_limit - time_since_last
             if self.verbose:
                 logger.debug(
-                    f"Rate limit triggered. Sleeping for {sleep_duration:.3f} seconds."
+                    f"Rate limit triggered (interval {self.rate_limit}s). Sleeping for {sleep_duration:.3f} seconds."
                 )
             time.sleep(sleep_duration)
 
@@ -610,23 +642,37 @@ class KarakeepAPI:
         self,
         archived: Optional[bool] = None,
         favourited: Optional[bool] = None,
-        limit: Optional[int] = None,
+        sort_order: Optional[Literal["asc", "desc"]] = None,
+        limit: Optional[int] = None,  # This is the per-page limit for the API
         cursor: Optional[str] = None,
         include_content: bool = True,  # Default from spec
-    ) -> Any:  # Returns PaginatedBookmarks or raw dict/list
+    ) -> Union[datatypes.PaginatedBookmarks, Dict[str, Any], List[Any]]:
         """
-        Get all bookmarks. Corresponds to GET /bookmarks.
+        Get bookmarks, one page at a time. Corresponds to GET /bookmarks.
+
+        This method fetches a single page of bookmarks.
+        The 'limit' parameter controls the number of items per page for this API call.
+        The 'cursor' parameter is used for pagination to get the next page.
+
+        CLI Usage Notes:
+        - When the 'get-all-bookmarks' command is used via the CLI:
+          - The `--cursor` CLI option is ignored; pagination is handled automatically.
+          - The `--limit` CLI option specifies the *total* number of bookmarks to fetch
+            across multiple pages. If omitted, all bookmarks are fetched.
+          - The CLI internally calls this API method multiple times to achieve this.
 
         Args:
             archived: Filter by archived status (optional).
             favourited: Filter by favourited status (optional).
-            limit: Maximum number of bookmarks to return (optional).
-            cursor: Pagination cursor for the next page (optional).
+            limit: Maximum number of bookmarks to return *per page* in this API call (optional).
+                   When used from the CLI's 'get-all-bookmarks' command, this translates to an
+                   internal per-page fetching limit, while the CLI's `--limit` controls the total.
+            cursor: Pagination cursor for fetching the next page (optional).
             include_content: If set to true, bookmark's content will be included (default: True).
 
         Returns:
-            datatypes.PaginatedBookmarks: Paginated list of bookmarks.
-            If response validation is disabled, returns the raw API response (dict/list).
+            datatypes.PaginatedBookmarks: Paginated list of bookmarks for the current page.
+            If response validation is disabled, returns the raw API response (dict/list) for the current page.
 
         Raises:
             APIError: If the API request fails.
@@ -635,6 +681,7 @@ class KarakeepAPI:
         params = {
             "archived": archived,
             "favourited": favourited,
+            "sortOrder": sort_order,
             "limit": limit,
             "cursor": cursor,
             "includeContent": include_content,  # Use camelCase as per API spec query param
@@ -671,7 +718,7 @@ class KarakeepAPI:
         fileName: Optional[str] = None,
         # size: Optional[int] = None, # Size is not in POST spec
         # content: Optional[str] = None, # Content is not in POST spec
-    ) -> Any:  # Returns Bookmark or raw dict/list
+    ) -> Union[datatypes.Bookmark, Dict[str, Any], List[Any]]:
         """
         Create a new bookmark. Corresponds to POST /bookmarks.
 
@@ -743,9 +790,7 @@ class KarakeepAPI:
                     "Argument 'asset_type' ('image' or 'pdf') is required when type is 'asset'."
                 )
             if assetId is None:
-                raise ValueError(
-                    "Argument 'assetId' is required when type is 'asset'."
-                )
+                raise ValueError("Argument 'assetId' is required when type is 'asset'.")
             request_body["assetType"] = asset_type
             request_body["assetId"] = assetId
             if fileName is not None:
@@ -770,15 +815,17 @@ class KarakeepAPI:
     def search_bookmarks(
         self,
         q: str,  # Search query is required
+        sort_order: Optional[Literal["asc", "desc", "relevance"]] = None,
         limit: Optional[int] = None,
         cursor: Optional[str] = None,
         include_content: bool = True,  # Default from spec
-    ) -> Any:  # Returns PaginatedBookmarks or raw dict/list
+    ) -> Union[datatypes.PaginatedBookmarks, Dict[str, Any], List[Any]]:
         """
         Search bookmarks. Corresponds to GET /bookmarks/search.
 
         Args:
             q: The search query string.
+            sort_order: Sort order for results ("asc", "desc", "relevance"). Default from API is "relevance" (optional).
             limit: Maximum number of bookmarks to return (optional).
             cursor: Pagination cursor for the next page (optional).
             include_content: If set to true, bookmark's content will be included (default: True).
@@ -793,6 +840,7 @@ class KarakeepAPI:
         """
         params = {
             "q": q,
+            "sortOrder": sort_order,
             "limit": limit,
             "cursor": cursor,
             "includeContent": include_content,  # Use camelCase as per API spec query param
@@ -809,7 +857,7 @@ class KarakeepAPI:
     @optional_typecheck
     def get_a_single_bookmark(
         self, bookmark_id: str, include_content: bool = True  # Default from spec
-    ) -> Any:  # Returns Bookmark or raw dict/list
+    ) -> Union[datatypes.Bookmark, Dict[str, Any], List[Any]]:
         """
         Get a single bookmark by its ID. Corresponds to GET /bookmarks/{bookmarkId}.
 
@@ -857,9 +905,7 @@ class KarakeepAPI:
         return None  # Explicitly return None for 204
 
     @optional_typecheck
-    def update_a_bookmark(
-        self, bookmark_id: str, update_data: dict
-    ) -> Any:  # Returns dict
+    def update_a_bookmark(self, bookmark_id: str, update_data: dict) -> Dict[str, Any]:
         """
         Update a bookmark by its ID. Corresponds to PATCH /bookmarks/{bookmarkId}.
         Allows updating fields like 'archived', 'favourited', 'summary', 'note', 'title', etc.
@@ -872,7 +918,7 @@ class KarakeepAPI:
         Returns:
             dict: A dictionary representing the updated bookmark (partial representation).
                   The response typically includes 'id', 'createdAt', 'modifiedAt', 'title', 'archived', 'favourited',
-                  'taggingStatus', 'note', 'summary'.
+                  'taggingStatus', 'summarizationStatus', 'note', 'summary'.
                   Validation is not performed on this response type by default.
 
         Raises:
@@ -885,7 +931,7 @@ class KarakeepAPI:
         return response_data
 
     @optional_typecheck
-    def summarize_a_bookmark(self, bookmark_id: str) -> Any:  # Returns dict
+    def summarize_a_bookmark(self, bookmark_id: str) -> Dict[str, Any]:
         """
         Summarize a bookmark by its ID. Corresponds to POST /bookmarks/{bookmarkId}/summarize.
         This triggers the summarization process and returns the updated bookmark record (partially).
@@ -909,16 +955,18 @@ class KarakeepAPI:
 
     @optional_typecheck
     def attach_tags_to_a_bookmark(
-        self, bookmark_id: str, tags_data: dict
-    ) -> Any:  # Returns dict
+        self,
+        bookmark_id: str,
+        tag_ids: Optional[List[str]] = None,
+        tag_names: Optional[List[str]] = None,
+    ) -> Dict[str, Any]:
         """
         Attach one or more tags to a bookmark. Corresponds to POST /bookmarks/{bookmarkId}/tags.
 
         Args:
             bookmark_id: The ID (string) of the bookmark.
-            tags_data: Dictionary specifying the tags to attach. Must contain a "tags" key
-                       which is a list of objects, each having *either* "tagId" (string) *or* "tagName" (string).
-                       Example: `{"tags": [{"tagId": "existing_tag_id"}, {"tagName": "new_or_existing_tag_name"}]}`
+            tag_ids: List of existing tag IDs to attach (optional).
+            tag_names: List of tag names to attach (will create tags if they don't exist) (optional).
 
         Returns:
             dict: A dictionary containing the list of attached tag IDs under the key "attached".
@@ -926,8 +974,50 @@ class KarakeepAPI:
                   Validation is not performed on this response type by default.
 
         Raises:
+            ValueError: If no tags are provided or if arguments are invalid.
             APIError: If the API request fails (e.g., 404 bookmark not found).
         """
+        # Validate that at least one tag source is provided
+        if not tag_ids and not tag_names:
+            raise ValueError("At least one of 'tag_ids' or 'tag_names' must be provided")
+
+        # Validate input types
+        if tag_ids is not None and not isinstance(tag_ids, list):
+            raise ValueError("'tag_ids' must be a list of strings")
+
+        if tag_names is not None and not isinstance(tag_names, list):
+            raise ValueError("'tag_names' must be a list of strings")
+
+        # Validate individual elements
+        if tag_ids:
+            for i, tag_id in enumerate(tag_ids):
+                if not isinstance(tag_id, str) or not tag_id.strip():
+                    raise ValueError(f"Tag ID at index {i} must be a non-empty string")
+
+        if tag_names:
+            for i, tag_name in enumerate(tag_names):
+                if not isinstance(tag_name, str) or not tag_name.strip():
+                    raise ValueError(f"Tag name at index {i} must be a non-empty string")
+
+        # Construct the tags_data dict in the format expected by the API
+        tags_list = []
+
+        if tag_ids:
+            for tag_id in tag_ids:
+                tags_list.append({"tagId": tag_id.strip()})
+
+        if tag_names:
+            for tag_name in tag_names:
+                tags_list.append({"tagName": tag_name.strip()})
+
+        tags_data = {"tags": tags_list}
+
+        # Optional validation using Tag datatype if validation is enabled
+        if not self.disable_response_validation:
+            # Validate the constructed structure matches expected format
+            # This is primarily for development/debugging purposes
+            logger.debug("Validating constructed tags_data structure")
+
         endpoint = f"bookmarks/{bookmark_id}/tags"
         response_data = self._call("POST", endpoint, data=tags_data)
         # Response schema is {"attached": [TagId]}, return as dict
@@ -936,16 +1026,18 @@ class KarakeepAPI:
 
     @optional_typecheck
     def detach_tags_from_a_bookmark(
-        self, bookmark_id: str, tags_data: dict
-    ) -> Any:  # Returns dict
+        self,
+        bookmark_id: str,
+        tag_ids: Optional[List[str]] = None,
+        tag_names: Optional[List[str]] = None,
+    ) -> Dict[str, Any]:
         """
         Detach one or more tags from a bookmark. Corresponds to DELETE /bookmarks/{bookmarkId}/tags.
 
         Args:
             bookmark_id: The ID (string) of the bookmark.
-            tags_data: Dictionary specifying the tags to detach. Must contain a "tags" key
-                       which is a list of objects, each having *either* "tagId" (string) *or* "tagName" (string).
-                       Example: `{"tags": [{"tagId": "tag_id_to_remove"}, {"tagName": "tag_name_to_remove"}]}`
+            tag_ids: List of existing tag IDs to detach (optional).
+            tag_names: List of tag names to detach (optional).
 
         Returns:
             dict: A dictionary containing the list of detached tag IDs under the key "detached".
@@ -953,8 +1045,50 @@ class KarakeepAPI:
                   Validation is not performed on this response type by default.
 
         Raises:
+            ValueError: If no tags are provided or if arguments are invalid.
             APIError: If the API request fails (e.g., 404 bookmark not found).
         """
+        # Validate that at least one tag source is provided
+        if not tag_ids and not tag_names:
+            raise ValueError("At least one of 'tag_ids' or 'tag_names' must be provided")
+
+        # Validate input types
+        if tag_ids is not None and not isinstance(tag_ids, list):
+            raise ValueError("'tag_ids' must be a list of strings")
+
+        if tag_names is not None and not isinstance(tag_names, list):
+            raise ValueError("'tag_names' must be a list of strings")
+
+        # Validate individual elements
+        if tag_ids:
+            for i, tag_id in enumerate(tag_ids):
+                if not isinstance(tag_id, str) or not tag_id.strip():
+                    raise ValueError(f"Tag ID at index {i} must be a non-empty string")
+
+        if tag_names:
+            for i, tag_name in enumerate(tag_names):
+                if not isinstance(tag_name, str) or not tag_name.strip():
+                    raise ValueError(f"Tag name at index {i} must be a non-empty string")
+
+        # Construct the tags_data dict in the format expected by the API
+        tags_list = []
+
+        if tag_ids:
+            for tag_id in tag_ids:
+                tags_list.append({"tagId": tag_id.strip()})
+
+        if tag_names:
+            for tag_name in tag_names:
+                tags_list.append({"tagName": tag_name.strip()})
+
+        tags_data = {"tags": tags_list}
+
+        # Optional validation using Tag datatype if validation is enabled
+        if not self.disable_response_validation:
+            # Validate the constructed structure matches expected format
+            # This is primarily for development/debugging purposes
+            logger.debug("Validating constructed tags_data structure")
+
         endpoint = f"bookmarks/{bookmark_id}/tags"
         response_data = self._call("DELETE", endpoint, data=tags_data)
         # Response schema is {"detached": [TagId]}, return as dict
@@ -964,7 +1098,7 @@ class KarakeepAPI:
     @optional_typecheck
     def get_highlights_of_a_bookmark(
         self, bookmark_id: str
-    ) -> Any:  # Returns List[Highlight] or raw dict/list
+    ) -> Union[List[datatypes.Highlight], Dict[str, Any], List[Any]]:
         """
         Get all highlights associated with a specific bookmark. Corresponds to GET /bookmarks/{bookmarkId}/highlights.
 
@@ -1012,7 +1146,7 @@ class KarakeepAPI:
     @optional_typecheck
     def attach_asset(
         self, bookmark_id: str, asset_data: dict
-    ) -> Any:  # Returns Asset or raw dict/list
+    ) -> Union[datatypes.Asset, Dict[str, Any], List[Any]]:
         """
         Attach a new asset to a bookmark. Corresponds to POST /bookmarks/{bookmarkId}/assets.
 
@@ -1020,7 +1154,7 @@ class KarakeepAPI:
             bookmark_id: The ID (string) of the bookmark.
             asset_data: Dictionary specifying the asset to attach. Must contain "id" (string) and "assetType" (string enum).
                         Example: `{"id": "asset_id_string", "assetType": "screenshot"}`
-                        See `datatypes.AssetType1` enum for possible asset types.
+                        See `datatypes.AssetType` enum for possible asset types.
 
         Returns:
             datatypes.Asset: The attached asset object.
@@ -1084,7 +1218,9 @@ class KarakeepAPI:
         return None  # Explicitly return None for 204
 
     @optional_typecheck
-    def get_all_lists(self) -> Any:  # Returns List[ListModel] or raw dict/list
+    def get_all_lists(
+        self,
+    ) -> Union[List[datatypes.ListModel], Dict[str, Any], List[Any]]:
         """
         Get all lists for the current user. Corresponds to GET /lists.
 
@@ -1127,24 +1263,53 @@ class KarakeepAPI:
 
     @optional_typecheck
     def create_a_new_list(
-        self, list_data: dict
-    ) -> Any:  # Returns ListModel or raw dict/list
+        self,
+        name: str,
+        icon: str,
+        description: Optional[str] = None,
+        parent_id: Optional[str] = None,
+        list_type: Optional[Literal["manual", "smart"]] = "manual",
+        query: Optional[str] = None,
+    ) -> Union[datatypes.ListModel, Dict[str, Any], List[Any]]:
         """
         Create a new list (manual or smart). Corresponds to POST /lists.
 
         Args:
-            list_data: Dictionary containing the data for the new list. Requires "name" (string) and "icon" (string).
-                       Optional fields include "description", "parentId", "type" ('manual' or 'smart'), "query".
-                       See the OpenAPI spec for details. Example: `{"name": "My List", "icon": "📚"}`
+            name: The name of the list (required, max 40 characters).
+            icon: The icon for the list (required).
+            description: Optional description for the list (max 100 characters).
+            parent_id: Optional parent list ID for nested lists.
+            list_type: The type of list ('manual' or 'smart'). Default is 'manual'.
+            query: Optional query string for smart lists (required if list_type is 'smart').
 
         Returns:
             datatypes.ListModel: The created list object.
             If response validation is disabled, returns the raw API response (dict/list).
 
         Raises:
+            ValueError: If required arguments are missing or invalid.
             APIError: If the API request fails (e.g., bad request, invalid data).
             pydantic.ValidationError: If response validation fails (and is not disabled).
         """
+        # Validate smart list requirements
+        if list_type == "smart" and not query:
+            raise ValueError("Argument 'query' is required when list_type is 'smart'.")
+
+        # Construct the request body
+        list_data = {
+            "name": name,
+            "icon": icon,
+            "type": list_type,
+        }
+
+        # Add optional fields if provided
+        if description is not None:
+            list_data["description"] = description
+        if parent_id is not None:
+            list_data["parentId"] = parent_id
+        if query is not None:
+            list_data["query"] = query
+
         response_data = self._call("POST", "lists", data=list_data)
 
         if self.disable_response_validation:
@@ -1157,7 +1322,7 @@ class KarakeepAPI:
     @optional_typecheck
     def get_a_single_list(
         self, list_id: str
-    ) -> Any:  # Returns ListModel or raw dict/list
+    ) -> Union[datatypes.ListModel, Dict[str, Any], List[Any]]:
         """
         Get a single list by its ID. Corresponds to GET /lists/{listId}.
 
@@ -1203,7 +1368,7 @@ class KarakeepAPI:
     @optional_typecheck
     def update_a_list(
         self, list_id: str, update_data: dict
-    ) -> Any:  # Returns ListModel or raw dict/list
+    ) -> Union[datatypes.ListModel, Dict[str, Any], List[Any]]:
         """
         Update a list by its ID. Corresponds to PATCH /lists/{listId}.
         Allows updating fields like "name", "description", "icon", "parentId", "query".
@@ -1232,18 +1397,20 @@ class KarakeepAPI:
             return datatypes.ListModel.model_validate(response_data)
 
     @optional_typecheck
-    def get_a_bookmarks_in_a_list(
+    def get_bookmarks_in_the_list(
         self,
         list_id: str,
+        sort_order: Optional[Literal["asc", "desc"]] = None,
         limit: Optional[int] = None,
         cursor: Optional[str] = None,
         include_content: bool = True,  # Default from spec
-    ) -> Any:  # Returns PaginatedBookmarks or raw dict/list
+    ) -> Union[datatypes.PaginatedBookmarks, Dict[str, Any], List[Any]]:
         """
         Get the bookmarks contained within a specific list. Corresponds to GET /lists/{listId}/bookmarks.
 
         Args:
             list_id: The ID (string) of the list.
+            sort_order: Sort order for results ("asc", "desc"). Default from API is "desc" (optional).
             limit: Maximum number of bookmarks to return (optional).
             cursor: Pagination cursor for the next page (optional).
             include_content: If set to true, bookmark's content will be included (default: True).
@@ -1258,6 +1425,7 @@ class KarakeepAPI:
         """
         endpoint = f"lists/{list_id}/bookmarks"
         params = {
+            "sortOrder": sort_order,
             "limit": limit,
             "cursor": cursor,
             "includeContent": include_content,  # Use camelCase as per API spec query param
@@ -1310,12 +1478,12 @@ class KarakeepAPI:
         return None  # Explicitly return None for 204
 
     @optional_typecheck
-    def get_all_tags(self) -> Any:  # Returns List[Tag1] or raw dict/list
+    def get_all_tags(self) -> Union[List[datatypes.Tag], Dict[str, Any], List[Any]]:
         """
         Get all tags for the current user. Corresponds to GET /tags.
 
         Returns:
-            List[datatypes.Tag1]: A list of tag objects, including bookmark counts.
+            List[datatypes.Tag]: A list of tag objects, including bookmark counts.
             If response validation is disabled, returns the raw API response (dict/list).
 
         Raises:
@@ -1329,7 +1497,7 @@ class KarakeepAPI:
             # Return raw data, which might be {"tags": [...]} or something else
             return response_data
         else:
-            # Response schema is {"tags": [Tag1]}, extract the list and validate
+            # Response schema is {"tags": [Tag]}, extract the list and validate
             if (
                 isinstance(response_data, dict)
                 and "tags" in response_data
@@ -1337,7 +1505,7 @@ class KarakeepAPI:
             ):
                 try:
                     return [
-                        datatypes.Tag1.model_validate(tag)
+                        datatypes.Tag.model_validate(tag)
                         for tag in response_data["tags"]
                     ]
                 except (
@@ -1352,7 +1520,37 @@ class KarakeepAPI:
                 )
 
     @optional_typecheck
-    def get_a_single_tag(self, tag_id: str) -> Any:  # Returns Tag1 or raw dict/list
+    def create_a_new_tag(self, name: str) -> Dict[str, Any]:
+        """
+        Create a new tag. Corresponds to POST /tags.
+
+        Args:
+            name: The name of the tag (required, minimum length 1).
+
+        Returns:
+            dict: A dictionary containing the created tag information with "id" and "name" fields.
+                  Validation is not performed on this response type by default.
+
+        Raises:
+            ValueError: If the name is empty or invalid.
+            APIError: If the API request fails (e.g., bad request, invalid data).
+        """
+        # Validate the name
+        if not name or not name.strip():
+            raise ValueError("Tag name cannot be empty.")
+
+        # Construct the request body
+        tag_data = {"name": name.strip()}
+
+        response_data = self._call("POST", "tags", data=tag_data)
+        # Response schema is a simple dict with id and name, return as dict
+        # No Pydantic validation applied here as the spec defines a simple dict response
+        return response_data
+
+    @optional_typecheck
+    def get_a_single_tag(
+        self, tag_id: str
+    ) -> Union[datatypes.Tag, Dict[str, Any], List[Any]]:
         """
         Get a single tag by its ID. Corresponds to GET /tags/{tagId}.
 
@@ -1360,7 +1558,7 @@ class KarakeepAPI:
             tag_id: The ID (string) of the tag to retrieve.
 
         Returns:
-            datatypes.Tag1: The requested tag object.
+            datatypes.Tag: The requested tag object.
             If response validation is disabled, returns the raw API response (dict/list).
 
         Raises:
@@ -1374,8 +1572,8 @@ class KarakeepAPI:
             logger.debug("Skipping response validation as requested.")
             return response_data
         else:
-            # Response should match Tag1 schema
-            return datatypes.Tag1.model_validate(response_data)
+            # Response should match Tag schema
+            return datatypes.Tag.model_validate(response_data)
 
     @optional_typecheck
     def delete_a_tag(self, tag_id: str) -> None:
@@ -1396,9 +1594,7 @@ class KarakeepAPI:
         return None  # Explicitly return None for 204
 
     @optional_typecheck
-    def update_a_tag(
-        self, tag_id: str, update_data: dict
-    ) -> Any:  # Returns Tag1 or raw dict/list
+    def update_a_tag(self, tag_id: str, update_data: dict) -> Dict[str, Any]:
         """
         Update a tag by its ID. Currently only supports updating the "name".
         Corresponds to PATCH /tags/{tagId}.
@@ -1409,41 +1605,33 @@ class KarakeepAPI:
                          Example: `{"name": "new tag name"}`
 
         Returns:
-            datatypes.Tag1: The updated tag object.
-            If response validation is disabled, returns the raw API response (dict/list).
+            dict: A dictionary containing the updated tag information with "id" and "name" fields.
+                  Validation is not performed on this response type by default.
 
         Raises:
             APIError: If the API request fails (e.g., 404 tag not found).
-            pydantic.ValidationError: If response validation fails (and is not disabled).
         """
         endpoint = f"tags/{tag_id}"
         response_data = self._call("PATCH", endpoint, data=update_data)
-
-        if self.disable_response_validation:
-            logger.debug("Skipping response validation as requested.")
-            return response_data
-        else:
-            # As of version 0.24.1 of karakeep: we do not check the correct
-            # validation type because there is an error on the
-            # server side: https://github.com/karakeep-app/karakeep/issues/1365
-            return response_data
-
-            # Response should match Tag1 schema
-            return datatypes.Tag1.model_validate(response_data)
+        # Response schema is a simple dict with id and name, return as dict
+        # No Pydantic validation applied here as the spec defines a simple dict response
+        return response_data
 
     @optional_typecheck
-    def get_a_bookmarks_with_the_tag(
+    def get_bookmarks_with_the_tag(
         self,
         tag_id: str,
+        sort_order: Optional[Literal["asc", "desc"]] = None,
         limit: Optional[int] = None,
         cursor: Optional[str] = None,
         include_content: bool = True,  # Default from spec
-    ) -> Any:  # Returns PaginatedBookmarks or raw dict/list
+    ) -> Union[datatypes.PaginatedBookmarks, Dict[str, Any], List[Any]]:
         """
         Get the bookmarks associated with a specific tag. Corresponds to GET /tags/{tagId}/bookmarks.
 
         Args:
             tag_id: The ID (string) of the tag.
+            sort_order: Sort order for results ("asc", "desc"). Default from API is "desc" (optional).
             limit: Maximum number of bookmarks to return (optional).
             cursor: Pagination cursor for the next page (optional).
             include_content: If set to true, bookmark's content will be included (default: True).
@@ -1458,6 +1646,7 @@ class KarakeepAPI:
         """
         endpoint = f"tags/{tag_id}/bookmarks"
         params = {
+            "sortOrder": sort_order,
             "limit": limit,
             "cursor": cursor,
             "includeContent": include_content,  # Use camelCase as per API spec query param
@@ -1474,7 +1663,7 @@ class KarakeepAPI:
     @optional_typecheck
     def get_all_highlights(
         self, limit: Optional[int] = None, cursor: Optional[str] = None
-    ) -> Any:  # Returns PaginatedHighlights or raw dict/list
+    ) -> Union[datatypes.PaginatedHighlights, Dict[str, Any], List[Any]]:
         """
         Get all highlights for the current user. Corresponds to GET /highlights.
 
@@ -1502,15 +1691,24 @@ class KarakeepAPI:
 
     @optional_typecheck
     def create_a_new_highlight(
-        self, highlight_data: dict
-    ) -> Any:  # Returns Highlight or raw dict/list
+        self,
+        bookmark_id: str,
+        start_offset: float,
+        end_offset: float,
+        color: Optional[Literal["yellow", "red", "green", "blue"]] = "yellow",
+        text: Optional[str] = None,
+        note: Optional[str] = None,
+    ) -> Union[datatypes.Highlight, Dict[str, Any], List[Any]]:
         """
         Create a new highlight on a bookmark. Corresponds to POST /highlights.
 
         Args:
-            highlight_data: Dictionary containing the data for the new highlight. Requires "bookmarkId" (string),
-                            "startOffset" (number), "endOffset" (number). Optional fields include "color", "text", "note".
-                            See the OpenAPI spec for details. Example: `{"bookmarkId": "...", "startOffset": 10, "endOffset": 25}`
+            bookmark_id: The ID of the bookmark to create the highlight on.
+            start_offset: The start position of the highlight in the text.
+            end_offset: The end position of the highlight in the text.
+            color: The color of the highlight ("yellow", "red", "green", "blue"). Default is "yellow".
+            text: Optional text content of the highlight.
+            note: Optional note associated with the highlight.
 
         Returns:
             datatypes.Highlight: The created highlight object.
@@ -1520,6 +1718,16 @@ class KarakeepAPI:
             APIError: If the API request fails (e.g., 400 bad request, 404 bookmark not found).
             pydantic.ValidationError: If response validation fails (and is not disabled).
         """
+        # Construct the request body
+        highlight_data = {
+            "bookmarkId": bookmark_id,
+            "startOffset": start_offset,
+            "endOffset": end_offset,
+            "color": color,
+            "text": text,
+            "note": note,
+        }
+
         response_data = self._call("POST", "highlights", data=highlight_data)
 
         if self.disable_response_validation:
@@ -1532,7 +1740,7 @@ class KarakeepAPI:
     @optional_typecheck
     def get_a_single_highlight(
         self, highlight_id: str
-    ) -> Any:  # Returns Highlight or raw dict/list
+    ) -> Union[datatypes.Highlight, Dict[str, Any], List[Any]]:
         """
         Get a single highlight by its ID. Corresponds to GET /highlights/{highlightId}.
 
@@ -1560,7 +1768,7 @@ class KarakeepAPI:
     @optional_typecheck
     def delete_a_highlight(
         self, highlight_id: str
-    ) -> Any:  # Returns Highlight or raw dict/list
+    ) -> Union[datatypes.Highlight, Dict[str, Any], List[Any]]:
         """
         Delete a highlight by its ID. Corresponds to DELETE /highlights/{highlightId}.
         Note: Unlike most DELETE endpoints, this returns the deleted highlight object on success (status 200).
@@ -1589,7 +1797,7 @@ class KarakeepAPI:
     @optional_typecheck
     def update_a_highlight(
         self, highlight_id: str, update_data: dict
-    ) -> Any:  # Returns Highlight or raw dict/list
+    ) -> Union[datatypes.Highlight, Dict[str, Any], List[Any]]:
         """
         Update a highlight by its ID. Currently only supports updating the "color".
         Corresponds to PATCH /highlights/{highlightId}.
@@ -1618,7 +1826,7 @@ class KarakeepAPI:
             return datatypes.Highlight.model_validate(response_data)
 
     @optional_typecheck
-    def get_current_user_info(self) -> Any:  # Returns dict
+    def get_current_user_info(self) -> Dict[str, Any]:
         """
         Get information about the current authenticated user. Corresponds to GET /users/me.
 
@@ -1634,7 +1842,7 @@ class KarakeepAPI:
         return response_data
 
     @optional_typecheck
-    def get_current_user_stats(self) -> Any:  # Returns dict
+    def get_current_user_stats(self) -> Dict[str, Any]:
         """
         Get statistics about the current authenticated user's data. Corresponds to GET /users/me/stats.
 
